@@ -1,6 +1,5 @@
-#  -*- coding: utf-8 -*-
 # SPDX-License-Identifier: MPL-2.0
-# Copyright 2020-2021 John Mille <john@compose-x.io>
+# Copyright 2020-2022 John Mille <john@compose-x.io>
 
 """
 Module for the ComposeXSettings class
@@ -17,27 +16,37 @@ import jsonschema
 import yaml
 from botocore.exceptions import ClientError
 from cfn_flip.yaml_dumper import LongCleanDumper
-from compose_x_common.aws import get_account_id, get_region_azs, validate_iam_role_arn
+from compose_x_common.aws import get_account_id, validate_iam_role_arn
 from compose_x_common.compose_x_common import keyisset
 from compose_x_render.compose_x_render import ComposeDefinition
 from importlib_resources import files as pkg_files
+from troposphere import AWSObject
 
 from ecs_composex import __version__
 from ecs_composex.common import LOG, NONALPHANUM
 from ecs_composex.common.aws import get_cross_role_session
-from ecs_composex.common.cfn_params import USE_FLEET_T
-from ecs_composex.common.compose_networks import ComposeNetwork
-from ecs_composex.common.compose_services import ComposeFamily, ComposeService
-from ecs_composex.common.compose_volumes import ComposeVolume
+from ecs_composex.compose.compose_networks import ComposeNetwork
+from ecs_composex.compose.compose_secrets import ComposeSecret
+from ecs_composex.compose.compose_services import ComposeService
+from ecs_composex.compose.compose_volumes import ComposeVolume
+from ecs_composex.compose.x_resources import XResource
+from ecs_composex.compose.x_resources.api_x_resources import ApiXResource
+from ecs_composex.compose.x_resources.environment_x_resources import (
+    AwsEnvironmentResource,
+)
+from ecs_composex.compose.x_resources.network_x_resources import NetworkXResource
+from ecs_composex.compose.x_resources.services_resources import ServicesXResource
+from ecs_composex.ecs.ecs_family import ComposeFamily
 from ecs_composex.iam import ROLE_ARN_ARG
-from ecs_composex.secrets.compose_secrets import ComposeSecret
 from ecs_composex.utils.init_ecs import set_ecs_settings
 from ecs_composex.utils.init_s3 import create_bucket
 
 
-class ComposeXSettings(object):
+class ComposeXSettings:
     """
     Class to handle the settings to use for ECS ComposeX.
+
+    :ivar dict of {str: ComposeFamily} families: Map of families and services
     """
 
     name_arg = "Name"
@@ -69,7 +78,6 @@ class ComposeXSettings(object):
     single_nat_arg = "SingleNat"
 
     default_vpc_cidr = "100.127.254.0/24"
-    default_azs = ["eu-west-1a", "eu-west-1b"]
     default_output_dir = f"/tmp/{dt.utcnow().strftime('%s')}"
 
     active_commands = [
@@ -124,10 +132,6 @@ class ComposeXSettings(object):
             if keyisset(self.region_arg, kwargs)
             else self.session.region_name
         )
-        self.aws_azs = self.default_azs
-        self.public_azs = self.default_azs
-        self.app_azs = self.default_azs
-        self.storage_azs = self.default_azs
 
         self.bucket_name = (
             None if not keyisset(self.bucket_arg, kwargs) else kwargs[self.bucket_arg]
@@ -136,9 +140,6 @@ class ComposeXSettings(object):
         self.services = []
         self.secrets = []
         self.networks = []
-        self.vpc_imported = False
-        self.subnets_parameters = []
-        self.subnets_mappings = {}
         self.secrets_mappings = {}
         self.mappings = {}
         self.families = {}
@@ -146,7 +147,6 @@ class ComposeXSettings(object):
         self.output_dir = self.default_output_dir
         self.format = self.default_format
 
-        self.create_vpc = False
         self.requires_private_namespace = False
         self.vpc_cidr = None
         self.single_nat = None
@@ -156,7 +156,6 @@ class ComposeXSettings(object):
         self.no_upload = True if keyisset(self.render_arg, kwargs) else False
 
         self.upload = False if self.no_upload else True
-        self.create_compute = False if not keyisset(USE_FLEET_T, kwargs) else True
         self.parse_command(kwargs, content)
         self.compose_content = {}
         self.input_file = (
@@ -168,30 +167,91 @@ class ComposeXSettings(object):
         self.evaluate_private_namespace()
         self.name = kwargs[self.name_arg]
         self.ecs_cluster = None
-        self.ecs_cluster_platform_override = None
-        self.ecs_cluster_providers = []
         self.ignore_ecr_findings = keyisset(self.ecr_arg, kwargs)
         self.x_resources_void = []
 
+    def get_x_resources(self, include_new=True, include_mappings=True) -> list:
+        """
+        Returns the x_resources with filters
+
+        :param include_new: Whether or not to add the new resources in the list
+        :param include_mappings: Whether or not to add the new resources in the listF
+        """
+        if not include_new and not include_mappings:
+            return self.x_resources
+        x_resources = []
+        for resource in self.x_resources:
+            if not include_new and (
+                resource.cfn_resource
+                and issubclass(type(resource.cfn_resource), AWSObject)
+            ):
+                continue
+            if not include_mappings and resource.mappings:
+                continue
+            x_resources.append(resource)
+        return x_resources
+
+    @property
+    def x_resources(self):
+        """
+        Iterates over all resources defined and returns the list of them
+        Only resources that are created from XResource(and children classes) are considered.
+        Avoids having to go through stacks down to resources and work backwards
+
+        Returns: the list of XResource in the execution.
+
+        """
+        all_keys = self.compose_content.keys()
+        all_resources = []
+        for res_key in all_keys:
+
+            if not isinstance(self.compose_content[res_key], dict):
+                continue
+            for resource in self.compose_content[res_key].values():
+                if not issubclass(
+                    type(resource),
+                    (
+                        XResource,
+                        ServicesXResource,
+                        NetworkXResource,
+                        ApiXResource,
+                        AwsEnvironmentResource,
+                    ),
+                ):
+                    continue
+                all_resources.append(resource)
+        return all_resources
+
     def evaluate_private_namespace(self):
         """
         Method to go over all services and figure out if any of them requires cloudmap.
         If so it will also expect x-dns.PrivateNamespace to be set.
         """
         self.requires_private_namespace = self.use_appmesh or any(
-            keyisset("UseCloudmap", service.x_network) for service in self.services
+            keyisset("x-cloudmap", service.x_network) for service in self.services
         )
         if self.requires_private_namespace:
             LOG.warning(
                 "At least one service requires cloudmap or AppMesh is used. Enabling private namespace"
             )
 
-    def set_secrets(self):
+    def requires_vpc(self):
+        """
+        Determines whether the execution will require a VPC.
+        """
+        x_resources_require_vpc = any([res.requires_vpc for res in self.x_resources])
+        services_require_vpc = any(
+            [
+                family.service_compute.launch_type != "EXTERNAL"
+                for family in self.families.values()
+                if (family and family.ecs_service)
+            ]
+        )
+        return any([x_resources_require_vpc, services_require_vpc])
+
+    def set_secrets(self) -> None:
         """
         Function to parse the settings compose content and define the secrets.
-
-        :param ecs_composex.common.settings.ComposeXSettings settings:
-        :return:
         """
         if not keyisset(ComposeSecret.main_key, self.compose_content):
             return
@@ -205,7 +265,7 @@ class ComposeXSettings(object):
                 self.secrets.append(secret)
                 self.compose_content[ComposeSecret.main_key][secret_name] = secret
 
-    def set_efs(self):
+    def set_efs(self) -> None:
         """
         Method to add a x-efs definition to the compose-x definition when a volume is flagged as using NFS/EFS
         """
@@ -233,7 +293,6 @@ class ComposeXSettings(object):
                         "Properties": volume.efs_definition,
                         "MacroParameters": volume.parameters,
                         "Lookup": volume.lookup,
-                        "Use": volume.use,
                         "Services": [
                             {"name": service.name, "access": "RW"}
                             for service in volume.services
@@ -246,7 +305,7 @@ class ComposeXSettings(object):
                         f"x-efs {volume.name} was already defined in top-level x-efs. Not overriding from volumes"
                     )
 
-    def set_volumes(self):
+    def set_volumes(self) -> None:
         """
         Method configuring the volumes at root level
         :return:
@@ -262,29 +321,24 @@ class ComposeXSettings(object):
             self.compose_content[ComposeVolume.main_key][volume_name] = volume
             self.volumes.append(volume)
 
-    def set_networks(self, vpc_stack, root_stack):
+    def set_networks(self, vpc_stack) -> None:
         """
-        Method configuring the networks defined at root level
-        :return:
+        Maps top level docker-compose networks with x-vpc subnets when applicable.
         """
         if not keyisset(ComposeNetwork.main_key, self.compose_content):
             LOG.debug("No networks detected at the root level of compose file")
             return
-        elif vpc_stack:
-            LOG.info(
-                "ComposeX will be creating the VPC, therefore networks are ignored!"
-            )
-            return
-        for network_name in self.compose_content[ComposeNetwork.main_key]:
+        networks = self.compose_content[ComposeNetwork.main_key]
+        for network_name, definition in networks.items():
             network = ComposeNetwork(
                 network_name,
-                self.compose_content[ComposeNetwork.main_key][network_name],
-                self.subnets_parameters,
+                definition,
+                vpc_stack.vpc_resource.subnets_parameters,
             )
-            self.compose_content[ComposeNetwork.main_key][network_name] = network
+            networks[network_name] = network
             self.networks.append(network)
 
-    def set_services(self):
+    def set_services(self) -> None:
         """
         Method to define the ComposeXResource for each service.
         :return:
@@ -301,9 +355,11 @@ class ComposeXSettings(object):
             self.compose_content[ComposeService.main_key][service_name] = service
             self.services.append(service)
 
-    def add_new_family(self, family_name, service, assigned_services):
-        if service.name in [r_service.name for r_service in assigned_services]:
-            LOG.info(
+    def add_new_family(
+        self, family_name: str, service: ComposeService, assigned_services: list
+    ) -> None:
+        if service in assigned_services:
+            print(
                 f"Detected {service.name} is-reused in different family. Making a deepcopy"
             )
             the_service = deepcopy(service)
@@ -315,24 +371,26 @@ class ComposeXSettings(object):
             family = ComposeFamily([service], family_name)
             service.my_family = family
         self.families[family.logical_name] = family
-        if service.name not in [service.name for service in assigned_services]:
+        if service not in assigned_services:
             assigned_services.append(service)
 
-    def handle_assigned_existing_service(self, family_name, service, assigned_services):
+    def add_service_to_family(
+        self, family_name: str, service: ComposeService, assigned_services: list
+    ) -> None:
         the_family = self.families[family_name]
-        if service.name in [r_service.name for r_service in assigned_services]:
+        if service in assigned_services:
             LOG.info(
-                f"Detected {service.name} is-reused in different family. Making a deepcopy"
+                f"Service {service.name} is-reused in different family. Making a deepcopy"
             )
-
             the_service = deepcopy(service)
-            the_family.add_service(the_service)
-            the_service.my_family = self.families[family_name]
-            self.services.append(the_service)
         else:
-            the_family.add_service(service)
-            service.my_family = self.families[family_name]
-            assigned_services.append(service)
+            the_service = service
+        LOG.debug(f"THE_SERVICE, {hex(id(the_service))}, SERVICE, {hex(id(service))}")
+        the_family.add_service(the_service)
+        the_service.my_family = the_family
+        self.services.append(the_service)
+        if the_service not in assigned_services:
+            assigned_services.append(the_service)
 
     def set_families(self):
         """
@@ -350,9 +408,10 @@ class ComposeXSettings(object):
                 if formatted_name not in self.families.keys():
                     self.add_new_family(family_name, service, assigned_services)
                 elif formatted_name in self.families.keys() and service.name not in [
-                    service.name for service in self.families[formatted_name].services
+                    _service.name
+                    for _service in self.families[formatted_name].ordered_services
                 ]:
-                    self.handle_assigned_existing_service(
+                    self.add_service_to_family(
                         formatted_name, service, assigned_services
                     )
         LOG.debug([self.families[family] for family in self.families])
@@ -373,7 +432,7 @@ class ComposeXSettings(object):
         content_def = ComposeDefinition(files, content)
         self.compose_content = content_def.definition
         source = pkg_files("ecs_composex").joinpath("specs/compose-spec.json")
-        print(source)
+        LOG.info(f"Validating against input schema {source}")
         resolver = jsonschema.RefResolver(
             f"file://{path.abspath(path.dirname(source))}/", None
         )
@@ -469,49 +528,10 @@ class ComposeXSettings(object):
             else self.default_output_dir
         )
 
-    def set_azs_from_api(self):
-        """
-        Method to set the AWS Azs based on DescribeAvailabilityZones
-        :return:
-        """
-        try:
-            self.aws_azs = get_region_azs(self.session)
-        except ClientError as error:
-            code = error.response["Error"]["Code"]
-            message = error.response["Error"]["Message"]
-            if code == "RequestExpired":
-                LOG.error(message)
-                LOG.warning(f"Due to error, using default values {self.aws_azs}")
-
-            else:
-                LOG.error(error)
-
-    def set_azs_from_vpc_import(self, subnets, session=None):
-        """
-        Function to get the list of AZs for a given set of subnets
-        :param dict subnets:
-        :param session: The Session used to find the EC2 subnets (useful for lookup).
-        :return:
-        """
-        if session is None:
-            client = self.session.client("ec2")
-        else:
-            client = session.client("ec2")
-        for subnet_name, subnet_definition in subnets.items():
-            if not isinstance(subnet_definition, list):
-                continue
-            try:
-                subnets_r = client.describe_subnets(SubnetIds=subnet_definition)[
-                    "Subnets"
-                ]
-                self.subnets_mappings[subnet_name]["Azs"] = [
-                    subnet["AvailabilityZone"] for subnet in subnets_r
-                ]
-            except ClientError:
-                LOG.warning("Could not define the AZs based on the imported subnets")
-        self.vpc_imported = True
-
     def set_bucket_name_from_account_id(self):
+        """
+        Defines the default bucket name to use from the AWS Account ID
+        """
         if self.bucket_name and isinstance(self.bucket_name, str):
             return
         if self.account_id is None:
